<?php
// $Id: Lock.php,v 1.1.2.2 2011/01/16 13:27:12 pounard Exp $

/**
 * Default implementation, and singleton implementation. Fine for most usages.
 */
class Oox_Lock
{
  /**
   * Default arbitrary lifetime for lock. 
   * 
   * @var int
   */
  const LIFETIME_DEFAULT = 30;

  /**
   * Singleton implementation.
   * 
   * @var Oox_Lock_Interface
   */
  private static $__instance;

  /**
   * Singleton implementation, get instance.
   * 
   * @return Oox_Lock_Interface
   */
  public static function getInstance() {
    if (!isset(self::$__instance)) {
      // Allow runtime implementation change. Some users might want to use
      // a cache based implementation (using memcache for example could be
      // a lot faster than use database queries).
      if ($class = variable_get(OOX_VAR_LOCK_BACKEND, 'Oox_Lock_Database')) {
        if (!oox_class_is($class, 'Oox_Lock_Interface')) {
          $class = 'Oox_Lock_Database';
          watchdog('oox', "Defined lock implementation " . $class . " does not implements the Oox_Lock_Interface interface", NULL, WATCHDOG_ERROR);
        }
        else {
          self::$__instance = new $class();
          if (!self::$__instance->checkEnvironment()) {
            unset(self::$__instance);
            watchdog('oox', "Your environement does not matches the " . $class . " implementation requirements", NULL, WATCHDOG_ERROR);
          }
        }
      }
      // Failsafe, if not instanciated upper, use the default implementation.
      if (!isset(self::$__instance)) {
        self::$__instance = new Oox_Lock_Database();
      }
    }
    return self::$__instance;
  }
}
