<?php

/**
 * Exception thrown when aptempting to acquire an already locked lock.
 */
class Oox_Lock_AlreadyAcquiredException extends Oox_Exception
{
  /**
   * Lock database row.
   * 
   * @var object
   */
  protected $_lock;

  /**
   * Current user static cache, since Drupal does not do cache into the
   * user_load() function.
   */
  protected $_user;

  /**
   * Get user which owns the current lock.
   * 
   * @return object
   *   Drupal database account row.
   */
  public function getAccount() {
    if (!isset($this->_user)) {
      if ($this->_lock->uid) {
        $this->_user = user_load($this->_lock->uid);
      }
      // Create a fake user, with a non existing uid, which name is
      // the current translation of "System".
      else {
        $this->_user = new stdClass();
        $this->_user->uid = -1;
        $this->_user->name = t("System");
        $this->_user->mail = variable_get('site_mail', '');
      }
    }
    return $this->_user;
  }

  /**
   * Get the lock database row.
   */
  public function getLock() {
    return $this->_lock;
  }

  /**
   * Build instance.
   * 
   * @param object $lock
   *   Lock database row.
   */
  public function __construct($lock) {
    parent::__construct("Already locked lock");
    $this->_lock = $lock;
  }
}
