<?php
// $Id: Optionable.php,v 1.1.2.1 2011/01/04 18:23:27 pounard Exp $

class Oox_Optionable_Dummy_Exception extends Exception {}

/**
 * Default implementation that can be used as base for buildind specific
 * implementation, either by inheritance, either by composition.
 */
class Oox_Optionable extends Oox_Registrable_Abstract implements Oox_Optionable_Interface
{
  /**
   * @var int
   */
  protected $_optionsIdentifier = NULL;

  /**
   * @see Oox_Optionable_Interface::getOptionsIdentifier()
   */
  public function getOptionsIdentifier() {
    return $this->_optionsIdentifier;
  }
  
  /**
   * @see Oox_Optionable_Interface::_setOptionsIdentifier()
   */
  public function _setOptionsIdentifier($oid) {
    $this->_optionsIdentifier = $oid;
    return $this;
  }

  /**
   * @var array
   */
  protected $_options = array();

  /**
   * @see Oox_Optionable_Interface::getOptionsCount()
   */
  public function getOptionsCount() {
    return count($this->_options);
  }

  /**
   * @see Oox_Optionable_Interface::setOptions()
   */
  public function setOptions(array $options, $reset = FALSE) {
    if ($reset) {
      $this->_options = $options;
    }
    else {
      _options_merge($this->_options, $options);
    }
    return $this;
  }

  /**
   * @see Oox_Optionable_Interface::getOptions()
   */
  public function getOptions() {
    return $this->_options;
  }

  /**
   * Recursively set an option using a multidimensional path.
   */
  public function _setRecursive(&$options, $path, $value) {
    $current = array_shift($path);
    if (empty($path)) {
      $options[$current] = $value;
    }
    else {
      $this->_setRecursive($options[$current], $path, $value);
    }
  }

  /**
   * @see Oox_Optionable_Interface::setOption()
   */
  public function setOption($name, $value) {
    if (strpos($name, Oox_Optionable_Interface::PATH_SEP)) {
      $this->_setRecursive(explode(Oox_Optionable_Interface::PATH_SEP, $name), $value);
    }
    else {
      $this->_options[$name] = $value;
    }
    return $this;
  }

  /**
   * Recursively get an option using a multidimensional path.
   */
  public function _getRecursive(&$options, $path, $default) {
    $current = array_shift($path);
    if (empty($path)) {
      if (isset($options[$current])) {
        return $options[$current];
      }
      else {
        // See comment on below method.
        throw new Oox_Optionable_Dummy_Exception();
      }
    }
    return $this->_getRecursive($options[$current], $path, $default);
  }

  /**
   * @see Oox_Optionable_Interface::getOption()
   */
  public function getOption($name, $default = NULL) {
    if (strpos($name, Oox_Optionable_Interface::PATH_SEP)) {
      // Exception based algorithm. If the PHP interpreter is smart enought, any
      // exception thrown will use a über setjmp() / longjmp() super really fast
      // algorithm then we are faster! But, I think I'm dreaming here. This
      // remains a cool algorithm so let's keep it for fun. Most implementations
      // will use the _options property though.
      try {
        $this->_getRecursive(explode(Oox_Optionable_Interface::PATH_SEP, $name), $default);
      }
      catch (Oox_Optionable_Dummy_Exception $e) {
        return $default;
      }
    }
    else if (isset($this->_options[$name])) {
      return $this->_options[$name];
    }
    return $default;
  }

  /**
   * @see Oox_Optionable_Interface::hasOption()
   */
  public function hasOption($name) {
    return isset($this->_options[$name]);
  }

  /**
   * @see Oox_Optionable_Interface::removeOption()
   */
  public function removeOption($name) {
    unset($this->_options[$name]);
    return $this;
  }

  /**
   * @see Oox_Exportable_Interface::getMapping()
   */
  public function getMapping() {
    return Oox_Exporter_Mapping::getInstance('options', $this->_optionsIdentifier, $properties = array('options' => Oox_Exporter_Mapping::getInstanceFromValue($this))); 
  }
}

/**
 * Helper function that merge arrays in a multidimensional way. Unlike the
 * array_merge_recursive() function, we will override distinct keys instead
 * of creating a new array dimension there.
 */
function _options_merge(&$to, $from) {
  foreach ($from as $key => $value) {
    // If bot are array, merge it gracefully.
    if (isset($to[$key]) && is_array($to[$key]) && is_array($value)) {
      _options_merge($to[$key], $value);
    }
    // In any case of conflict (one is an array but not the other) the $from
    // array wins, and we override the current value with the new one.
    else {
      $to[$key] = $value;
    }
  }
}
