<?php

/**
 * Define an object that can be used in registries.
 * 
 * Any object can be used in registries, this particular ones can have a type
 * property, usefull for dynamic processing using machine names instead of
 * testing instances classes.
 */
interface Oox_Registrable_Interface
{
  /**
   * Set the module that carries this specific implementation. This is an
   * internal method and should never be used outside the registry itself.
   * 
   * @param string $module
   *   Module machine name.
   * 
   * @return Oox_Registrable_Interface
   *   Self reference for chaining.
   */
  public function _setModule($module);

  /**
   * Get the module that carries this specific object implementation.
   * 
   * @return string
   *   Module machine name.
   */
  public function getModule();

  /**
   * Public, but internal method, for registries. At instanciation time, the
   * registry will set the internal registry type using this method.
   * 
   * @param string $type
   *   Registry machine name.
   * 
   * @return Oox_Registrable_Interface
   *   Self reference for chaining.
   */
  public function _setRegistryType($type);

  /**
   * Get the internal registry type (defined as key in the oox registry
   * hook).
   * 
   * @return string
   *   Registry machine name.
   */
  public function getRegistryType();

  /**
   * Fetch the registry that created this object instance.
   * 
   * @return Oox_Registry
   */
  public function getRegistry();

  /**
   * Public, but internal method, for registries. At instanciation time, the
   * registry will set the internal type using this method.
   * 
   * @param string $type
   *   Class machine name.
   * 
   * @return Oox_Registrable_Interface
   *   Self reference for chaining.
   */
  public function _setType($type);

  /**
   * Get the internal object type (defined as key in the registry hook).
   * 
   * @return string
   *   Class machine name.
   */
  public function getType();
}
