<?php

/**
 * Base implementation.
 */
abstract class Formable_Abstract implements Formable_Interface
{
  /**
   * Default implementation will implecitely try to use class named such as:
   *   - <Parent_Class>_Form for default form;
   *   - <Parent_Class>_<Form_Identifier>_Form for others.
   * 
   * If you use this implementation, all you have to do is to create any of
   * these classes. If you want the system to be able to list your forms,
   * you can override the getFormIdentifiers() method as well.
   * 
   * @see Formable_Interface::getForm()
   */
  public function getForm($formIdentifier = Formable_Interface::FORM_DEFAULT) {

    if ($formIdentifier != Formable_Interface::FORM_DEFAULT) {
      $className = get_class($this) . '_' . ucfirst($formIdentifier) . '_Form';
    }
    else {
      $className = get_class($this) . '_Form';
    }

    if (class_exists($className)) {
      return new $className;
    }
    else {
      throw new Formable_Exception("Class '" . $className . "' does not exits.");
    }
  }

  /**
   * @see Formable_Interface::getFormIdentifiers()
   */
  public function getFormIdentifiers() { return array(); }
}
