<?php

/**
 * Most modules should probably use this implementation.
 * 
 * As the interface was meant to be used mainly for registrable objects, we
 * hopefully can set parents from there. For most cases, this would be more
 * than enough.
 * 
 * "Orphan" formable instances, which are not being processed by any process
 * helper that take care of it should then populate parents at every step of
 * the form building, at process, validate and submit.
 */
abstract class Formable_Form_Abstract implements Formable_Form_Interface
{
  /**
   * @var array
   */
  protected $_parents = array();

  /**
   * @see Formable_Form_Interface::getParents()
   */
  public function setParents(array $parents) {
    $this->_parents = $parents;
    return $this;
  }

  /**
   * @see Formable_Form_Interface::getParents()
   */
  public function getParents() {
    return $this->_parents;
  }

  /**
   * Considering that forms are being built with a set of complex elements
   * carrying their own validation methods, this function won't be used too
   * often by specific implementations.
   * 
   * We can safely assume that we probably don't need it in most cases.
   * 
   * @see Formable_Form_Interface::formValidate()
   */
  public function formValidate(array &$values, $object = NULL) {}
}