<?php

interface Formable_Form_Interface
{
  /**
   * Set Form API parents array, if set.
   * 
   * @param array $parents
   * 
   * @return Formable_Form_Interface
   *   Self reference for chaining.
   */
  public function setParents(array $parents);

  /**
   * Get Form API parents array, if set.
   * 
   * @return array
   */
  public function getParents();

  /**
   * Build the form.
   * 
   * The builded form may be a global form subset, with #tree property always
   * set, so please care about this.
   * 
   * @param array &$form
   *   This is the base form for this element. In most cases it will be an
   *   empty array. All you have to do is populate this variable with your
   *   own elements.
   * @param array $values = array()
   *   (optional) You can receive this array in misc. contextes. The most
   *   common one is when the form is being rebuilt using AJAX/AHAH form.
   *   If you receive those values, use it to override your own values or
   *   change the form shape if needed.
   * @param mixed $object = NULL
   *   (optional) Parent object. If form comes from a Formable_Interface
   *   implementor, then this instance will be passed here.
   */
  public function formBuild(array &$form, array $values = array(), $object = NULL);

  /**
   * Validate the form.
   * 
   * Values are the value subset corresponding to the form() method returned
   * elements.
   * 
   * @param array &$values
   *   Form values, can be an empty array if form() method did not returned
   *   any elements.
   *
   * @return array
   *   Multidimensional array. Path in array is the form element path, relative
   *   to form() function. Values are localized error messages.
   *   If no errors, either an empty array either NULL can be returned. This
   *   means that overriding methods should always test for this return type
   *   when calling the parent method.
   * @param mixed $object = NULL
   *   (optional) Parent object. If form comes from a Formable_Interface
   *   implementor, then this instance will be passed here.
   */
  public function formValidate(array &$values, $object = NULL);

  /**
   * Submit the form, in most cases you are working inside a persistent object,
   * just update its values. The underlaying form that embed your own has to
   * really save whatever in database itself.
   * 
   * Values are the value subset corresponding to the form() method returned
   * elements.
   * 
   * @param array &$values
   *   Form values, can be an empty array if form() method did not returned
   *   any elements.
   * @param mixed $object = NULL
   *   (optional) Parent object. If form comes from a Formable_Interface
   *   implementor, then this instance will be passed here.
   */
  public function formSubmit(array &$values, $object = NULL);
}
