<?php

/**
 * Helper form form handling.
 */
class Oox_Formable_Helper
{
  /**
   * Internal helper.
   */
  protected static function _mergeRecursive(array &$form, array $values) {
//    foreach ($values as $key => $value) {
//      if (isset($form[$key])) {
//        return TRUE;
//      }
//      else {
//        return FALSE;
//      }
//    }
  }

  /**
   * Set errors using form_set_error() in the current form being validated.
   * 
   * This function is for form developers that need to embed an object form in
   * a classic Drupal form.
   * 
   * @param array $errors
   *   Array of errors returned by Oox_Formable_Interface::formValidate().
   * @param array $originalPath = array()
   *   (optional) Current FAPI path as an array of element names.
   */
  public static function formSetErrors(array $errors, array $pathPrefix = array()) {
    foreach ($errors as $path => $error) {
      // Compute local path.
      $currentPath = $pathPrefix;
      $currentPath[] = $path;
      // If we have not an array here, this means we reached a real error.
      if (!is_array($error)) {
        form_set_error(implode('][', $currentPath), $error);
      }
      // An array here mean we didn't reach the last level element. Recurse
      // deeper then.
      else {
        self::formSetErrors($error, $currentPath);
      }
    }
  }

  /**
   * Specialized version of Oox_Formable_Helper::formSetErrors() that will set
   * errors relative to current given element instead of the full form.
   * 
   * @param array $errors
   *   Array of errors returned by Oox_Formable_Interface::formValidate().
   * @param array &$element
   *   Current FAPI element.
   */
  public static function elementSetErrors(array $errors, array &$element) {
    // Shortcut.
    if (empty($errors)) {
      return;
    }
    self::formSetErrors($errors, array_values($element['#parents']));
  }

  /**
   * Helper for implementors that does not implement AJAX/AHAH handling. This
   * function will reset your own elements values / default values using the
   * given one.
   * 
   * Form can be #tree'ed, and values can be a multidimensional array, all will
   * be safely merged, and nothing will be done in case of conflict between the
   * two variables structure.
   * 
   * @param array &$form
   *   Fully built form.
   * @param array $values
   *   Value array.
   */
  public static function applyValues(array &$form, array $values) {
    self::_mergeRecursive($form, $values);
  }
}
