<?php

/**
 * This interface represents any kind of objects that can be configured using
 * a form. Notice this is a really global 'configuration' concept.
 * 
 * If you need to do any other form that a form used to configure the object
 * itself, then consider building your own Drupal form in you custom module
 * instead of trying to use this.
 * 
 * This will mainly be used by common administration pages code, in order to
 * include objects implementation specific form additions for objects editing.
 */
interface Formable_Interface
{
  /**
   * Default form identifier.
   */
  const FORM_DEFAULT = 'default';

  /**
   * Get form helper instance.
   * 
   * @param $formIdentifier = Oox_Formable_Interface::FORM_DEFAULT
   *   (optional) Form identifier.
   * 
   * @return Formable_Form_Interface
   */
  public function getForm($formIdentifier = Formable_Interface::FORM_DEFAULT);

  /**
   * Get supported forms.
   * 
   * @return array
   *   Array of string name, including or not the
   *   Formable_Interface::FORM_DEFAULT constant.
   */
  public function getFormIdentifiers();
}
