<?php

/**
 * Active configuration objects are meant to implement the Active Record
 * pattern, and all data should be save while modifying it.
 * 
 * While this is not a necessity they did it or not, the new override()
 * method can help for improving performances in some cases.
 */
interface Schema_Active_Interface extends Schema_Interface
{
  /**
   * Remove definitely a single entry from the permanent storage, if exists.
   * 
   * @param string $name
   *   Valid path.
   */
  public function remove($name);

  /**
   * From the given schema, remove all values that matches in the current
   * instance.
   * 
   * Removing means deleting them definitely if they are being saved into
   * a permenent storage backend.
   * 
   * @param Schema_Readonly_Interface $schema
   */
  public function removeAll(Schema_Readonly_Interface $schema);

  /**
   * Merge current object with given one, forcing all new data to superseed the
   * already set one, but without triggering any real save into database.
   * 
   * This function aims to be used for local or temporary runtime overrides.
   */
  public function override(Schema_Readonly_Interface $schema);
}
