<?php

/**
 * Default implementation based on an internal array. Should be enough for most
 * implementations.
 */
class Schema_Array implements Schema_Interface
// FIXME: Temporary.
 , Countable, Iterator
{
  /**
   * Is this instance readonly.
   * 
   * @var bool
   */
  protected $_readonly;

  /**
   * Strict mode will force exception to raise when reaching non existing
   * configuration keys. Can be useful for debuging.
   * 
   * @var bool
   */
  protected $_strict;

  /**
   * Current section data.
   * 
   * @var array
   */
  protected $_data = array();

  /**
   * Find element pointed by the given path.
   * 
   * FIXME: De-recursification would greatly improve performances here.
   */
  protected function _findElement(array $path, array $done = array()) {
    $done[] = $key = array_shift($path);

    if (!array_key_exists($key, $this->_data)) {
      if (!$this->_strict) {
        return NULL;
      }
      else {
        throw new Schema_Exception("'" . implode('.', $done) . "' does not exists");
      }
    }

    if (count($path)) {
      if ($this->_data[$key] instanceof Schema_Interface) {
        return $this->_data[$key]->_findElement($path, $done);
      }
      else {
        throw new Schema_Exception("'" . implode('.', $done) . "' is not a value therefore we cannot browse further");
      }
    }
    else {
      return $this->_data[$key];
    }
  }

  /**
   * @see Schema_Readonly_Interface::get()
   */
  public function get($name) {
    $element = $this->_findElement(Schema_Helper::getPathFromName($name));

    if (isset($element) && !is_scalar($element)) {
      throw new Schema_Exception("'" . $name . "' is not a scalar value");
    }

    return $element;
  }

  /**
   * Set element pointed by the given path.
   * 
   * FIXME: De-recursification would greatly improve performances here.
   */
  protected function _setElement($value, array $path, array $done = array()) {
    $done[] = $key = array_shift($path);

    if (count($path)) {
      if (is_scalar($this->_data[$key])) {
        throw new Schema_Exception("Attempting to replace scalar value '" . implode('.', $done) . "' with a section.");
      }
      else if (!$this->_data[$key] instanceof Schema_Interface) {
        $this->_data[$key] = new Schema_Array(array());
      }
      return $this->_data[$key]->_setElement($value, $path, $done);
    }
    else {
      return $this->_data[$key] = $value;
    }
  }

  /**
   * @see Schema_Interface::set()
   */
  public function set($name, $value) {
    if ($this->_readonly) {
      throw new Schema_Exception("This schema is readonly");
    }

    $this->_setElement($value, Schema_Helper::getPathFromName($name));
  }

  /**
   * @see Schema_Readonly_Interface::getSection()
   */
  public function getSection($name) {
    $element = $this->_findElement(Schema_Helper::getPathFromName($name));

    if (!$element instanceof Schema_Interface) {
      throw new Schema_Exception("'" . $name . "' is a value");
    }

    return $element;
  }

  /**
   * @see Schema_Interface::setSection()
   */
  public function setSection($name, Schema_Readonly_Interface $section) {
    if ($this->_readonly) {
      throw new Schema_Exception("This schema is readonly");
    }

    $this->_setElement($section, Schema_Helper::getPathFromName($name));
  }

  /**
   * @see Schema_Interface::merge()
   */
  public function merge(Schema_Readonly_Interface $schema, $override = FALSE) {
    if ($this->_readonly) {
      throw new Schema_Exception("This schema is readonly");
    }

    foreach ($schema as $name => $value) {
      if ($value instanceof Schema_Readonly_Interface) {
        if (isset($this->_data[$name])) {
          if ($this->_data[$name] instanceof self) {
            $this->_data[$name]->merge($value, $override);
          }
          else {
            throw new Schema_Exception("Could not merge a value with a section.");
          }
        }
        else {
          $this->_data[$name] = new self($value->toArray(), $this->_readonly, $this->_strict);
        }
      }
      else if ($override || !isset($this->_data[$name])) {
        $this->_data[$name] = $value;
      }
    }
  }

  /**
   * @see Countable::count()
   */
  public function count() {
    return count($this->_data);
  }

  /**
   * Current data count cache, useful for Iterator interface.
   * 
   * @var int
   */
  protected $_count;

  /**
   * Current search offset, useful for Iterator interface.
   * 
   * @var 
   */
  protected $_offset;

  /**
   * @see Iterator::current()
   */
  public function current() {
    return current($this->_data);
  }

  /**
   * @see Iterator::next()
   */
  public function next() {
    $this->_offset++;
    return next($this->_data);
  }

  /**
   * @see Iterator::key()
   */
  public function key() {
    return key($this->_data);
  }

  /**
   * @see Iterator::valid()
   */
  public function valid() {
    return $this->_offset < $this->_count;
  }

  /**
   * @see Iterator::rewind()
   */
  public function rewind() {
    $this->_count = count($this->_data);
    $this->_offset = 0;
    reset($this->_data);
  }

  /**
   * @see Schema_Readonly_Interface::toArray()
   */
  public function toArray() {
    $array = array();
    $data = $this->_data;
    foreach ($data as $key => $value) {
      if ($value instanceof Schema_Interface) {
        $array[$key] = $value->toArray();
      }
      else {
        $array[$key] = $value;
      }
    }
    return $array;
  }

  /**
   * Forces a deep clone when cloning.
   */
  public function __clone() {
    $array = array();
    foreach ($this->_data as $key => $value) {
      if ($value instanceof Schema_Interface) {
        $array[$key] = clone $value;
      }
      else {
        $array[$key] = $value;
      }
    }
    $this->_data = $array;
  }

  /**
   * Default constructor.
   * 
   * @param mixed $data = array()
   *   Must be an array or an Iterator instance.
   * @param bool $readonly = FALSE
   *   If set to TRUE, the object won't be modifiable.
   * @param bool $strict = TRUE
   *   If set to TRUE, this object will raise exception when reaching non
   *   existing pathes.
   */
  public function __construct($data = array(), $readonly = FALSE, $strict = TRUE) {
    $this->_readonly = $readonly;
    $this->_strict = $strict;
    foreach ($data as $key => $value) {
      if ($value instanceof Iterator || is_array($value)) {
        $this->_data[$key] = new self($value, $readonly, $strict);
      }
      else {
        $this->_data[$key] = $value;
      }
    }
  }
}
