<?php

/**
 * Define an object that can export its own configuration as a readonly schema
 * this will allow third party module to make quite easily exportable objects.
 * 
 * It does not provide an object creation UI, therefore it's your responsability
 * to do it. An object building mecanism based on a schema broswing should be
 * trivial to make, therefore I'm not worried about letting people do their own
 * sauce here.
 */
interface Schema_Exportable_Interface
{
  /**
   * Get configuration mapping.
   * 
   * Better for you to implement this by cloning the actual schema if already
   * instanciated. You cannot control what will do the developer that will
   * manipulate your schema.
   * 
   * @return Schema_Readonly_Interface
   */
  public function getSchema();

  /**
   * Rebuild the object internals using the given schema.
   * 
   * @param Schema_Interface $config
   *   Configuration object, if provided. The object must restore itself
   *   fully using the given options.
   * 
   * @return Oox_Registrable_Interface
   *   Self reference for chaining.
   */
  public function restoreFromSchema(Schema_Interface $config);
}
