<?php

/**
 * Collection of static helper for Schema handling.
 */
class Schema_Helper
{
  /**
   * Path separator.
   */
  const SEP = '.';

  /**
   * Get path parts as an array.
   * 
   * @param string $name
   *   A path such as: "site.information.name"
   *   One deep path is invalid.
   * 
   * @return array
   *   Path parts.
   * 
   * @throws Schema_Exception
   *   If path is malformed.
   */
  public static function getPathFromName($name, $deep = FALSE) {
    $path = explode(self::SEP, $name);

    if ($deep) {
      if (2 > count($path)) {
        throw new Schema_Exception("Path is invalid");
      }
    }
    else if (1 > count($path)) {
      throw new Schema_Exception("Path is invalid");
    }

    return $path;
  }

  /**
   * Check if path is valid.
   * 
   * @return bool
   */
  public static function isPathValid($name, $deep = FALSE) {
    // A lot more efficient than a single preg_match() call.
    if (($pos = strpos($name, ".")) !== FALSE) {
      return (0 < $pos) && ($pos < (strlen($name) - 1));
    }
    // No dot is still a valid path.
    return !$deep;
  }
}
