<?php

/**
 * Schema interface.
 */
interface Schema_Interface extends Schema_Readonly_Interface
{
  /**
   * Add a Schemauration variable.
   *
   * @param string $name
   *   A path such as: "site.information.name"
   *   One deep path is invalid.
   * 
   * @param mixed $value
   *   Value typed depending on the Schema definition.
   * 
   * @throws Schema_Exception
   *   If path is invalid.
   *   If path is a section.
   *   If in debug mode and path does not exists.   
   */
  public function set($name, $value);

  /**
   * Set a substree of the current schema.
   * 
   * @param string $name
   *   A path such as "site.information"
   *   A path leading to a value will fail.
   *
   * @param Schema_Readonly_Interface $section
   * 
   * @throws Schema_Exception
   *   If path is invalid.
   *   If path is a value.
   */
  public function setSection($name, Schema_Readonly_Interface $section);

  /**
   * Merge current object with given one.
   * 
   * @param Schema_Readonly_Interface $schema
   * @param bool $override = FALSE
   *   If set to TRUE, local existing values will be replaced by new ones.
   */
  public function merge(Schema_Readonly_Interface $schema, $override = FALSE);
}
