<?php

/**
 * Base implementation that uses Drupal core stream wrapper interfaces or a
 * local file path for reading a plain file.
 */
abstract class Schema_Reader_StreamWrapper extends Schema_Reader_String
{
  /**
   * @var string
   */
  protected $_uri;

  /**
   * @see Schema_Reader_String::read()
   */
  public function read() {
    if (!isset($this->_buffer)) {
      // Attempt to find if file does exists if the scheme is invalid. This is
      // best to do it here so readers can be instanciated without doing any
      // I/O.
      // If you use a reader with the default Schema_Cached_Array class, as
      // long as the actual cache is enough (most read operations on a stable
      // system running from a long run time won't need to refresh their cache)
      // the read() method won't be run. Therefore, testing it only when needed
      // ensure the system can run on cache entries only without running
      // exceptions even if the file system is downgraded or damaged.
      if (!file_stream_wrapper_valid_scheme($uri) && !file_exists($this->_uri)) {
        throw new Schema_Exception("Attempt to use a stream wrapper based schema source file from an invalid URI or a non existing file: '" . $this->_uri . "'.");
      }
      else {
        $this->_buffer = file_get_contents($this->_uri);
      }
    }

    return parent::read();
  }

  /**
   * Default constructor.
   * 
   * @param string $uri
   *   Valid URI.
   */
  public function __construct($uri) {
    $this->_uri = $uri;
  }
}
