<?php

/**
 * This interface is the smallest one you can use.
 * 
 * Any exportable object that will be able to export its own description as a
 * schema won't need it to be readable, therefore this small interface will be
 * enough for them to implement (or use one of the default implementations).
 */
interface Schema_Readonly_Interface /* extends Countable, Iterator */
{
  /**
   * Get variable.
   * 
   * @param string $name
   *   A path such as: "site.information.name"
   *   One deep path is invalid.
   * @param bool $default = FALSE
   *   If set to TRUE, fetch system default instead of current value.
   * 
   * @throws Schema_Exception
   *   If path is invalid.
   *   If path is a section.
   *   If in debug mode and path does not exists.
   */
  public function get($name);

  /**
   * Get a subtree of the current schema.
   * 
   * Avoid this calling this method for any other reasons that administration
   * tasks, it will bypass most caches in place therefore will result in an
   * idiotically slow code.
   * 
   * @param string $name
   *   A path such as "site.information"
   *   A path leading to a value will fail, as well as an empty path.
   * 
   * @return Schema_Readonly_Interface
   *   A Schema_Interface instance, which can be typed otherwise than this
   *   object class, depending on Schemauration merges that could have
   *   occured during execution time.
   * 
   * @throws Schema_Exception
   *   If path is invalid.
   *   If path is a value.
   *   If section does not exits.   
   */
  public function getSection($name);

  /**
   * Export the current configuration object as raw array. This array can
   * contain only scalar values or other similar arrays.
   * 
   * @return array
   */
  public function toArray();
}
