<?php

/**
 * @file
 * Oox Admin UI pages.
 */

/**
 * Global overview form, can display either a named objects listing, or an
 * integer identified objects listing.
 */
function oox_ui_registry_overview($form, &$form_state) {
  $form['#theme'] = 'oox_ui_registry_overview';

  foreach (Oox_Registry::_getAllDescriptions() as $registryType => $desc) {
    // FIXME: Weird bug, but it happens..
    if (empty($registryType)) {
      continue;
    }

    $registry = Oox_Registry::getInstance($registryType);

    $form['registries'][$registryType]['type'] = array('#markup' => $registryType);
    $form['registries'][$registryType]['name'] = array(
      '#prefix' => '<strong>',
      '#suffix' => '</strong>',
      '#markup' => isset($desc['name']) ? t($desc['name']) : $registryType,
    );
    $form['registries'][$registryType]['description'] = array(
      '#prefix' => '<small>',
      '#suffix' => '</small>',
      '#markup' => isset($desc['description']) ? t($desc['description']) : '<em>' . t("No description") . '</em>',
    );
    $form['registries'][$registryType]['item_count'] = array('#markup' => '<strong>' . count($registry->getItemCache()) . '</strong>');
    $modules = system_list('module_enabled');
    $form['registries'][$registryType]['module'] = array('#markup' => isset($modules[$desc['module']]) ? t($modules[$desc['module']]->info['name']) : '<em>' . t("Unknown") . '</em>');

    // Operations.
    $links = array();
    $links[] = array(
      'href' => 'admin/objects/registry/' . $registryType,
      'title' => t('Items'),
    );
    // XoXo module is tied to this one, and one without the other may have
    // really few chances of existing.
    /*
     * FIXME: No logical explaination, I must have lost mee brain!
     * 
    if (module_exists('xoxo')) {
      try {
        $form['registries'][$registryType]['persistant'] = array('#markup' => $registry->getRegistry()->getType() == 'storage' ? t("Yes") : t("No"));
      }
      catch (Oox_Registry_Exception $e) {
        // An exception here would mean that our registry does not have a
        // parent, which is totally legal (and probably will be for most
        // of them).
        $form['registries'][$registryType]['persistant'] = array('#markup' => t("No"));
      }
    }
     */

    $form['registries'][$registryType]['operations'] = array('#markup' => theme('links', array('links' => $links)));
  }

  return $form;
}

/**
 * Global registries configuration form.
 */
function oox_ui_registry_manage($form, &$form_state, $registry) {
  $form['#tree'] = TRUE;
  $form['#theme'] = 'oox_ui_registry_manage';

  // Prepare form.
  $form['table'] = array();
  $form['weights'] = array();
  $form['#registryType'] = $registryType = $registry->getType();

  // Populate weights table.
  foreach ($registry->getItemCache() as $itemType => $desc) {
    // Set first weight to 0.
    $currentWeight = 0;
    // Set form values.
    $form['table'][$itemType]['type'] = array('#markup' => $itemType);
    $form['table'][$itemType]['name'] = array(
      '#prefix' => '<strong>',
      '#suffix' => '</strong>',
      '#markup' => t($registry->getItemName($itemType)),
    );
    if (isset($desc['description'])) {
      $form['table'][$itemType]['description'] = array(
        '#prefix' => '<small>',
        '#suffix' => '</small>',
        '#markup' =>  t($desc['description']),
      );
    }
    else {
      $form['table'][$itemType]['description'] = array(
        '#prefix' => '<small><em>',
        '#suffix' => '</em></small>',
        '#markup' =>  t("No description"),
      );
    }
    $modules = system_list('module_enabled');
    $form['table'][$itemType]['module'] = array('#markup' => isset($modules[$desc['module']]) ? t($modules[$desc['module']]->info['name']) : '<em>' . t("Unknown") . '</em>');

    // Operations.
    $links = array();
    $links[] = array(
      'href' => 'admin/objects/registry/' . $registryType . '/' . $itemType,
      'title' => t('Info'),
    );
    // XoXo module is tied to this one, and one without the other may have
    // really few chances of existing. This hardcoded module existence check
    // may be ugly, but at least, it's pretty dawn cool integration.
    if (module_exists('xoxo_entity') && ($info = xoxo_entity_get_info($registryType, $itemType))) {
      $links[] = array(
        'href' => $info['admin']['real path'] . '/fields',
        'title' => t('Fields'),
      );
      $links[] = array(
        'href' => $info['admin']['real path'] . '/display',
        'title' => t('Display'),
      );
    }
    $form['table'][$itemType]['operations'] = array('#markup' => theme('links', array('links' => $links)));

    $form['weights'][$itemType] = array('#type' => 'hidden', '#default_value' => $currentWeight++);
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t("Save configuration"),
  );

  return $form;
}

/**
 * Submit handler for global registries configuration.
 */
function oox_ui_registry_manage_submit($form, &$form_state) {
  $registry = Oox_Registry::getInstance($form['#registryType']);
  foreach ($form_state['values']['weights'] as $type => $data) {
//    try {
//      variable_set($registry->_getHookName() . '_weight', $data);
//      // Explicitely clear and rebuild item cache.
//      $registry->_rebuildItemCache();
//    }
//    catch (Oox_Exception $e) {
//      watchdog_exception('oox', $e, NULL);
//      watchdog('oox', "Error while saving new weights for registry '" . $type . "'", NULL, WATCHDOG_ERROR);
//      drupal_set_message(t("An internal error happened, please consult watchdog"), 'error');
//    }
  }
  drupal_set_message(t("Registry item weights have been changed."));
}

/**
 * Manage registry item.
 */
function oox_ui_registry_item_manage($form, &$form_state, $registyItem, Oox_Registry $registry) {
  $form['properties_title'] = array('#markup' => '<h2>' . t("Object properties") . '</h2>');
  $rows = array();
  $desc = $registry->getItemCache($registyItem);
  foreach ($registry->getProperties() as $key => $name) {
    if (isset($desc['properties'][$key])) {
      $propertyValue = '<strong>' . (is_array($desc['properties'][$key]) ? implode(', ', $desc['properties'][$key]) : $desc['properties'][$key]) . '</strong>';
    }
    else {
      $propertyValue = '<em>' . t("Not set") . '</em>';
    }
    $rows[] = array(t($name), $propertyValue);
  }
  $form['properties_summary']['#markup'] = theme('table', array('rows' => $rows));
  $form['data_title'] = array('#markup' => '<h2>' . t("Raw object") . '</h2>');
  $form['data_summary'] = array(
    '#prefix' => '<pre>',
    '#suffix' => '</pre>',
    '#markup' => print_r($registyItem, TRUE),
  );
  return $form;
}

/**
 * Global configuration form.
 */
function oox_ui_common_settings($form, &$form_state) {
  $form = array();

  $form['actions']['rebuild'] = array(
    '#type' => 'submit',
    '#value' => t("Rebuild object cache"),
    '#attributes' => array(
      'title' => t("This will rebuild the object cache without flushing the full Drupal cache. Notice that it will also clear the menu cache in order for items to display correctly in administration pages. It can help in some edge cases where your cache did not rebuild correctly."),
    ),
    '#submit' => array('oox_ui_common_settings_submit_rebuild'),
  );

  return $form;
}

function oox_ui_common_settings_submit_rebuild($form, &$form_state) {
  cache_clear_all('*', OOX_CACHE_TABLE, TRUE);
  menu_rebuild();
  drupal_set_message(t("Object cache cleared."));
}

/**
 * Theme function for registries overview form.
 */
function theme_oox_ui_registry_overview(&$variables) {
  $form = &$variables['form'];
  drupal_add_css(drupal_get_path('module', 'oox_ui') . '/oox_ui.admin.css');

  // Prepare table rendering.
  $headers = array(t("Name"), t("Type"), /* t("Description"), */ t("Items"), t("Module"), t("Operations"));
  $rows = array();

  foreach (element_children($form['registries']) as $key) {
    $row = array();
    $row['data'][] = array(
      'title' => $form['registries'][$key]['description']['#markup'],
      'data' => drupal_render($form['registries'][$key]['name']),
    );
    $row['data'][] = drupal_render($form['registries'][$key]['type']);
    //$row['data'][] = drupal_render($form['registries'][$key]['description']);
    $row['data'][] = drupal_render($form['registries'][$key]['item_count']);
    $row['data'][] = drupal_render($form['registries'][$key]['module']);
    $row['data'][] = drupal_render($form['registries'][$key]['operations']);
    $rows[] = $row;
    // Remove description.
    unset($form['registries'][$key]['description']);
  }

  return theme('table', array(
    'header' => $headers,
    'attributes' => array('class' => array('oox-table')),
    'rows' => $rows,
  )) . drupal_render_children($form);
}

/**
 * Theme for registries weights form.
 */
function theme_oox_ui_registry_manage(&$variables) {
  $form = &$variables['form'];
  drupal_add_css(drupal_get_path('module', 'oox_ui') . '/oox_ui.admin.css');

  // Prepare table.
  $headers = array(t("Name"), t("Item type"), t("Module"), t("Operations"));
  $rows = array();

  // Iterate on items descriptions.
  foreach (element_children($form['table']) as $itemType) {
    $form['weights'][$itemType]['#attributes']['class'] = array('registry-item-order-weight');
    $rows[] = array(
      'data' => array(
        array(
         'data' => drupal_render($form['table'][$itemType]['name']) . drupal_render($form['weights'][$itemType]),
         'title' => $form['table'][$itemType]['description']['#markup'],
        ),
        drupal_render($form['table'][$itemType]['type']),
        drupal_render($form['table'][$itemType]['module']),
        drupal_render($form['table'][$itemType]['operations']),
      ),
      'class' => array('draggable'),
    );
    unset($form['table'][$itemType]['description']);
  }

  // Compute arbitrary CSS identifier for the current table.
  $key = 'registry-order';

  // Put the table into a fieldset.
  drupal_add_tabledrag($key, 'order', 'sibling', 'registry-item-order-weight', NULL, NULL, FALSE);

  // Final rendering.
  return theme('table', array(
    'rows' => $rows,
    'attributes' => array('id' => $key, 'class' => array('oox-table')),
    'header' => $headers,
  )) . drupal_render_children($form);
}
